/*
 * hushexport
 * 
 * Licensed under the MIT License
 * 
 * Copyright (c) 2012 NGTV (ngtv.fdempa.com)  / hush / ngtv.hsn@gmail.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 */
package com.fdempa.ngtv.hushexport
{
	import com.adobe.images.PNGEncoder;

	import flash.display.BitmapData;
	import flash.display.DisplayObjectContainer;
	import flash.display.MovieClip;
	import flash.errors.IOError;
	import flash.events.Event;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.utils.ByteArray;
	import flash.utils.getTimer;

	public class PNGSequence
	{
		private var _root:MovieClip;
		private var _folderName:String;
		private var _startFrame:int;
		private var _endFrame:int;
		public function PNGSequence(r:MovieClip, folderName:String, startFrame:int = 1, endFrame:int = 0)
		{
			_root = r;
			_folderName = folderName;
			_startFrame = (startFrame <= _root.totalFrames) ? startFrame : _root.totalFrames;
			_endFrame = endFrame || _root.totalFrames;
			_root.addEventListener(Event.ENTER_FRAME, _initialHandler);
		}
		private function init():void
		{
			_root.gotoAndStop(1);
			//MCの再生位置を1フレームから再生したように進める
			while(_root.currentFrame < _startFrame)
			{
				_root.nextFrame();
				nextFrameAllChildren(_root);
			}
			captureAllImage();
		}
		private function _initialHandler(e:Event):void 
		{
			//ここまで来たらロード完了らしい
			//Memo  http://wiki.minaco.net/index.php?Flash%2F%E3%83%89%E3%82%AD%E3%83%A5%E3%83%A1%E3%83%B3%E3%83%88%E3%82%AF%E3%83%A9%E3%82%B9%E8%B6%85%E8%A7%A3%E8%AA%AC
			if(_root.currentFrame == 3)
			{
				_root.removeEventListener(Event.ENTER_FRAME, _initialHandler);
				init();
			}
		}
		private function captureAllImage():void
		{
			captureImage(); //1フレームだけ別処理
			do{
				_root.nextFrame();
				nextFrameAllChildren(_root);
				captureImage();
			}while((_root.currentFrame != _root.totalFrames)&&
				   (_root.currentFrame < _endFrame));
		}
		private function traceFrames():void
		{
			trace(_root.currentFrame + "/" +_root.totalFrames +":" + getTimer());
		}

		private function captureImage():void
		{
			var canvas:BitmapData = new BitmapData(_root.stage.stageWidth, _root.stage.stageHeight, false);
			canvas.draw(_root.stage);
			var ba:ByteArray = PNGEncoder.encode(canvas);
			canvas.dispose();
			saveImage("pic"+getNumberString(_root.currentFrame)+".png",ba);
			traceFrames();
		}

		private function getNumberString(n:int):String
		{
			return String(100000+n).substring(1);
		}

		private function saveImage(filename:String, ba:ByteArray):void
		{
			var file:File = File.desktopDirectory.resolvePath(_folderName+"/"+filename);
			var stream:FileStream = new FileStream();
			try{
				stream.open(file, FileMode.WRITE);
				stream.writeBytes(ba);
			} catch (error:IOError) {
				trace(error.message);
			} finally {
				stream.close();
			}
		}

		private function nextFrameAllChildren(obj:DisplayObjectContainer):void
		{
			var i:int;
			var num = obj.numChildren;
			for(i = 0;i<num;i++)
			{
				if(obj.getChildAt(i) is DisplayObjectContainer){
					// 再帰しましょう
					nextFrameAllChildren(obj.getChildAt(i) as DisplayObjectContainer);
				}
				if(obj.getChildAt(i) is MovieClip){
					(obj.getChildAt(i) as MovieClip).nextFrame();
				}
			}
		}

	}
}

